@@ -20,7 +20,7 @@ if confighelp then
       [[
 redis_history {
   # History key name
-  key_prefix = 'rs_history';
+  key_prefix = 'rs_history{{HOSTNAME}}{{COMPRESS}}';
   # History expire in seconds
   expire = 0;
   # History rows limit
@@ -49,12 +49,15 @@ local ucl = require "ucl"
 local ts = (require "tableshape").types
 local E = {}
 local N = "history_redis"
-local hostname = rspamd_util.get_hostname()
+
+local template_env = {
+  HOSTNAME = rspamd_util.get_hostname(),
+}
 
 local redis_params
 
 local settings = {
-  key_prefix = 'rs_history', -- default key name
+  key_prefix = 'rs_history{{HOSTNAME}}{{COMPRESS}}', -- default key name template
   expire = nil, -- default no expire
   nrows = 200, -- default rows limit
   compress = true, -- use zstd compression when storing data in redis
@@ -64,7 +67,7 @@ local settings = {
   subject_privacy_length = 16, -- cut the length of the hash
 }
 
-local settings_schema = ts.shape({
+local settings_schema = lua_redis.enrich_schema({
   key_prefix = ts.string,
   expire = (ts.number + ts.string / lua_util.parse_time_interval):is_optional(),
   nrows = ts.number,
@@ -73,7 +76,7 @@ local settings_schema = ts.shape({
   subject_privacy_alg = ts.string:is_optional(),
   subject_privacy_prefix = ts.string:is_optional(),
   subject_privacy_length = ts.number:is_optional(),
-}, {extra_fields = lua_redis.config_schema})
+})
 
 local function process_addr(addr)
   if addr then
@@ -89,12 +92,17 @@ local function normalise_results(tbl, task)
   -- Convert stupid metric object
   if metric then
     tbl.symbols = {}
-    local symbols, others = fun.partition(function(k, v)
+    local symbols, others = fun.partition(function(_, v)
       return type(v) == 'table' and v.score
     end, metric)
 
-    fun.each(function(k, v) v.name = nil; tbl.symbols[k] = v; end, symbols)
-    fun.each(function(k, v) tbl[k] = v end, others)
+    fun.each(function(k, v)
+      v.name = nil;
+      tbl.symbols[k] = v;
+    end, symbols)
+    fun.each(function(k, v)
+      tbl[k] = v
+    end, others)
 
     -- Reset the original metric
     tbl.default = nil
@@ -109,6 +117,7 @@ local function normalise_results(tbl, task)
   tbl.rmilter = nil
   tbl.messages = nil
   tbl.urls = nil
+  tbl.action = task:get_metric_action()
 
   local seconds = task:get_timeval()['tv_sec']
   tbl.unix_time = seconds
@@ -139,8 +148,8 @@ local function history_save(task)
     return
   end
 
-  local data = task:get_protocol_reply{'metrics', 'basic'}
-  local prefix = settings.key_prefix .. hostname
+  local data = task:get_protocol_reply { 'metrics', 'basic' }
+  local prefix = lua_util.jinja_template(settings.key_prefix, template_env, false, true)
 
   if data then
     normalise_results(data, task)
@@ -148,57 +157,51 @@ local function history_save(task)
     rspamd_logger.errx('cannot get protocol reply, skip saving in history')
     return
   end
-  -- 1 is 'json-compact' but faster
-  local json = ucl.to_format(data, 1)
+
+  local json = ucl.to_format(data, 'json-compact')
 
   if settings.compress then
     json = rspamd_util.zstd_compress(json)
-    -- Distinguish between compressed and non-compressed options
-    prefix = prefix .. '_zst'
   end
 
   local ret, conn, _ = lua_redis.rspamd_redis_make_request(task,
-    redis_params, -- connect params
-    nil, -- hash key
-    true, -- is write
-    redis_llen_cb, --callback
-    'LPUSH', -- command
-    {prefix, json} -- arguments
+      redis_params, -- connect params
+      nil, -- hash key
+      true, -- is write
+      redis_llen_cb, --callback
+      'LPUSH', -- command
+      { prefix, json } -- arguments
   )
 
   if ret then
-    conn:add_cmd('LTRIM', {prefix, '0', string.format('%d', settings.nrows-1)})
+    conn:add_cmd('LTRIM', { prefix, '0', string.format('%d', settings.nrows - 1) })
 
     if settings.expire and settings.expire > 0 then
-      conn:add_cmd('EXPIRE', {prefix, string.format('%d', settings.expire)})
+      conn:add_cmd('EXPIRE', { prefix, string.format('%d', settings.expire) })
     end
   end
 end
 
 local function handle_history_request(task, conn, from, to, reset)
-  local prefix = settings.key_prefix .. hostname
-  if settings.compress then
-    -- Distinguish between compressed and non-compressed options
-    prefix = prefix .. '_zst'
-  end
+  local prefix = lua_util.jinja_template(settings.key_prefix, template_env, false, true)
 
   if reset then
     local function redis_ltrim_cb(err, _)
       if err then
         rspamd_logger.errx(task, 'got error %s when resetting history: %s',
-          err)
+            err)
         conn:send_error(504, '{"error": "' .. err .. '"}')
       else
         conn:send_string('{"success":true}')
       end
     end
     lua_redis.rspamd_redis_make_request(task,
-      redis_params, -- connect params
-      nil, -- hash key
-      true, -- is write
-      redis_ltrim_cb, --callback
-      'LTRIM', -- command
-      {prefix, '0', '0'} -- arguments
+        redis_params, -- connect params
+        nil, -- hash key
+        true, -- is write
+        redis_ltrim_cb, --callback
+        'LTRIM', -- command
+        { prefix, '0', '0' } -- arguments
     )
   else
     local function redis_lrange_cb(err, data)
@@ -209,14 +212,16 @@ local function handle_history_request(task, conn, from, to, reset)
         if settings.compress then
           local t1 = rspamd_util:get_ticks()
 
-          data = fun.totable(fun.filter(function(e) return e ~= nil end,
-            fun.map(function(e)
-              local _,dec = rspamd_util.zstd_decompress(e)
-              if dec then
-                return dec
-              end
-              return nil
-            end, data)))
+          data = fun.totable(fun.filter(function(e)
+            return e ~= nil
+          end,
+              fun.map(function(e)
+                local _, dec = rspamd_util.zstd_decompress(e)
+                if dec then
+                  return dec
+                end
+                return nil
+              end, data)))
           lua_util.debugm(N, task, 'decompress took %s ms',
               (rspamd_util:get_ticks() - t1) * 1000.0)
           collectgarbage()
@@ -224,23 +229,25 @@ local function handle_history_request(task, conn, from, to, reset)
         -- Parse elements using ucl
         local t1 = rspamd_util:get_ticks()
         data = fun.totable(
-          fun.map(function (_, obj) return obj end,
-          fun.filter(function(res, obj)
-              if res then
-                return true
-              end
-              return false
+            fun.map(function(_, obj)
+              return obj
             end,
-            fun.map(function(elt)
-              local parser = ucl.parser()
-              local res,_ = parser:parse_text(elt)
+                fun.filter(function(res, obj)
+                  if res then
+                    return true
+                  end
+                  return false
+                end,
+                    fun.map(function(elt)
+                      local parser = ucl.parser()
+                      local res, _ = parser:parse_text(elt)
 
-              if res then
-                return true, parser:get_object()
-              else
-                return false, nil
-              end
-            end, data))))
+                      if res then
+                        return true, parser:get_object()
+                      else
+                        return false, nil
+                      end
+                    end, data))))
         lua_util.debugm(N, task, 'parse took %s ms',
             (rspamd_util:get_ticks() - t1) * 1000.0)
         collectgarbage()
@@ -252,26 +259,26 @@ local function handle_history_request(task, conn, from, to, reset)
         collectgarbage()
       else
         rspamd_logger.errx(task, 'got error %s when getting history: %s',
-          err)
+            err)
         conn:send_error(504, '{"error": "' .. err .. '"}')
       end
     end
     lua_redis.rspamd_redis_make_request(task,
-      redis_params, -- connect params
-      nil, -- hash key
-      false, -- is write
-      redis_lrange_cb, --callback
-      'LRANGE', -- command
-      {prefix, string.format('%d', from), string.format('%d', to)}, -- arguments
-      {opaque_data = true}
+        redis_params, -- connect params
+        nil, -- hash key
+        false, -- is write
+        redis_lrange_cb, --callback
+        'LRANGE', -- command
+        { prefix, string.format('%d', from), string.format('%d', to) }, -- arguments
+        { opaque_data = true }
     )
   end
 end
 
-local opts =  rspamd_config:get_all_opt('history_redis')
+local opts = rspamd_config:get_all_opt('history_redis')
 if opts then
   settings = lua_util.override_defaults(settings, opts)
-  local res,err = settings_schema:transform(settings)
+  local res, err = settings_schema:transform(settings)
 
   if not res then
     rspamd_logger.warnx(rspamd_config, '%s: plugin is misconfigured: %s', N, err)
@@ -280,6 +287,12 @@ if opts then
   end
   settings = res
 
+  if settings.compress then
+    template_env.COMPRESS = '_zst'
+  else
+    template_env.COMPRESS = ''
+  end
+
   redis_params = lua_redis.parse_redis_server('history_redis')
   if not redis_params then
     rspamd_logger.infox(rspamd_config, 'no servers are specified, disabling module')
@@ -290,9 +303,9 @@ if opts then
       type = 'idempotent',
       callback = history_save,
       flags = 'empty,explicit_disable,ignore_passthrough',
-      augmentations = {string.format("timeout=%f", redis_params.timeout or 0.0)}
+      augmentations = { string.format("timeout=%f", redis_params.timeout or 0.0) }
     })
-    lua_redis.register_prefix(settings.key_prefix .. hostname, N,
+    lua_redis.register_prefix(lua_util.jinja_template(settings.key_prefix, template_env, false, true), N,
         "Redis history", {
           type = 'list',
         })
