@@ -26,9 +26,12 @@ local lua_util = require "lua_util"
 local rspamd_http = require "rspamd_http"
 local rspamd_util = require "rspamd_util"
 local rspamd_logger = require "rspamd_logger"
+local rspamd_tcp = require "rspamd_tcp"
+local lua_redis = require "lua_redis"
 local ucl = require "ucl"
 local E = {}
 local N = 'metadata_exporter'
+local HOSTNAME = rspamd_util.get_hostname()
 
 local settings = {
   pusher_enabled = {},
@@ -61,6 +64,7 @@ Message-ID: $message_id
 Action: $action
 Score: $score
 Symbols: $symbols]],
+  timeout = 5.0,
 }
 
 local function get_general_metadata(task, flatten, no_content)
@@ -74,15 +78,16 @@ local function get_general_metadata(task, flatten, no_content)
   r.user = task:get_user() or 'unknown'
   r.qid = task:get_queue_id() or 'unknown'
   r.subject = task:get_subject() or 'unknown'
-  r.action = task:get_metric_action('default')
+  r.action = task:get_metric_action()
+  r.rspamd_server = HOSTNAME
 
-  local s = task:get_metric_score('default')[1]
+  local s = task:get_metric_score()[1]
   r.score = flatten and string.format('%.2f', s) or s
 
   local fuzzy = task:get_mempool():get_variable("fuzzy_hashes", "fstrings")
   if fuzzy and #fuzzy > 0 then
     local fz = {}
-    for _,h in ipairs(fuzzy) do
+    for _, h in ipairs(fuzzy) do
       table.insert(fz, h)
     end
     if not flatten then
@@ -151,6 +156,20 @@ local function get_general_metadata(task, flatten, no_content)
       return 'unknown'
     end
   end
+
+  local scan_real = task:get_scan_time()
+  scan_real = math.floor(scan_real * 1000)
+  if scan_real < 0 then
+    rspamd_logger.messagex(task,
+        'clock skew detected for message: %s ms real sca time (reset to 0)',
+        scan_real)
+    scan_real = 0
+  end
+
+  r.scan_time = scan_real
+  local content = task:get_content()
+  r.size = content and content:len() or 0
+
   if not no_content then
     r.header_from = process_header('from')
     r.header_to = process_header('to')
@@ -177,7 +196,7 @@ local formatters = {
     else
       for _, e in ipairs(mail_rcpt) do
         table.insert(display_emails, string.format('<%s>', e))
-        table.insert(mail_targets, mail_rcpt)
+        table.insert(mail_targets, e)
       end
     end
     if rule.email_alert_sender then
@@ -208,7 +227,7 @@ local formatters = {
     meta.mail_to = table.concat(display_emails, ', ')
     meta.our_message_id = rspamd_util.random_hex(12) .. '@rspamd'
     meta.date = rspamd_util.time_to_string(rspamd_util.get_time())
-    return lua_util.template(rule.email_template or settings.email_template, meta), { mail_targets = mail_targets}
+    return lua_util.template(rule.email_template or settings.email_template, meta), { mail_targets = mail_targets }
   end,
   json = function(task)
     return ucl.to_format(get_general_metadata(task), 'json-compact')
@@ -224,29 +243,29 @@ local selectors = {
     return true
   end,
   is_spam = function(task)
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return is_spam(action)
   end,
   is_spam_authed = function(task)
     if not task:get_user() then
       return false
     end
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return is_spam(action)
   end,
   is_reject = function(task)
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return (action == 'reject')
   end,
   is_reject_authed = function(task)
     if not task:get_user() then
       return false
     end
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return (action == 'reject')
   end,
   is_not_soft_reject = function(task)
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return (action ~= 'soft reject')
   end,
 }
@@ -260,7 +279,7 @@ end
 
 local pushers = {
   redis_pubsub = function(task, formatted, rule)
-    local _,ret,upstream
+    local _, ret, upstream
     local function redis_pub_cb(err)
       if err then
         rspamd_logger.errx(task, 'got error %s when publishing on server %s',
@@ -269,13 +288,13 @@ local pushers = {
       end
       return true
     end
-    ret,_,upstream = rspamd_redis_make_request(task,
-      redis_params, -- connect params
-      nil, -- hash key
-      true, -- is write
-      redis_pub_cb, --callback
-      'PUBLISH', -- command
-      {rule.channel, formatted} -- arguments
+    ret, _, upstream = lua_redis.redis_make_request(task,
+        redis_params, -- connect params
+        nil, -- hash key
+        true, -- is write
+        redis_pub_cb, --callback
+        'PUBLISH', -- command
+        { rule.channel, formatted } -- arguments
     )
     if not ret then
       rspamd_logger.errx(task, 'error connecting to redis')
@@ -284,15 +303,19 @@ local pushers = {
   end,
   http = function(task, formatted, rule)
     local function http_callback(err, code)
+      local valid_status = { 200, 201, 202, 204 }
+
       if err then
         rspamd_logger.errx(task, 'got error %s in http callback', err)
         return maybe_defer(task, rule)
       end
-      if code ~= 200 then
-        rspamd_logger.errx(task, 'got unexpected http status: %s', code)
-        return maybe_defer(task, rule)
+      for _, v in ipairs(valid_status) do
+        if v == code then
+          return true
+        end
       end
-      return true
+      rspamd_logger.errx(task, 'got unexpected http status: %s', code)
+      return maybe_defer(task, rule)
     end
     local hdrs = {}
     if rule.meta_headers then
@@ -302,17 +325,21 @@ local pushers = {
         if type(v) == 'table' then
           hdrs[pfx .. k] = ucl.to_format(v, 'json-compact')
         else
-          hdrs[pfx .. k] = v
+          hdrs[pfx .. k] = rspamd_util.mime_header_encode(v)
         end
       end
     end
+
     rspamd_http.request({
-      task=task,
-      url=rule.url,
-      body=formatted,
-      callback=http_callback,
-      mime_type=rule.mime_type or settings.mime_type,
-      headers=hdrs,
+      task = task,
+      url = rule.url,
+      user = rule.user,
+      password = rule.password,
+      body = formatted,
+      callback = http_callback,
+      mime_type = rule.mime_type or settings.mime_type,
+      headers = hdrs,
+      timeout = rule.timeout or settings.timeout,
     })
   end,
   send_mail = function(task, formatted, rule, extra)
@@ -334,10 +361,30 @@ local pushers = {
       timeout = rule.timeout or settings.timeout,
     }, formatted, sendmail_cb)
   end,
+  json_raw_tcp = function(task, formatted, rule)
+    local function json_raw_tcp_callback(err, code)
+      if err then
+        rspamd_logger.errx(task, 'got error %s in json_raw_tcp callback', err)
+        return maybe_defer(task, rule)
+      end
+      return true
+    end
+    rspamd_tcp.request({
+      task = task,
+      host = rule.host,
+      port = rule.port,
+      data = formatted,
+      callback = json_raw_tcp_callback,
+      timeout = rule.timeout or settings.timeout,
+      read = false,
+    })
+  end,
 }
 
 local opts = rspamd_config:get_all_opt(N)
-if not opts then return end
+if not opts then
+  return
+end
 local process_settings = {
   select = function(val)
     selectors.custom = assert(load(val))()
@@ -459,7 +506,7 @@ if type(settings.rules) ~= 'table' then
     end
   end
   if settings.pusher_enabled.redis_pubsub then
-    redis_params = rspamd_parse_redis_server(N)
+    redis_params = lua_redis.parse_redis_server(N)
     if not redis_params then
       rspamd_logger.errx(rspamd_config, 'No redis servers are specified')
       settings.pusher_enabled.redis_pubsub = nil
@@ -470,6 +517,7 @@ if type(settings.rules) ~= 'table' then
       r.defer = settings.defer
       r.selector = settings.pusher_select.redis_pubsub
       r.formatter = settings.pusher_format.redis_pubsub
+      r.timeout = redis_params.timeout
       settings.rules[r.backend:upper()] = r
     end
   end
@@ -507,6 +555,21 @@ if type(settings.rules) ~= 'table' then
       settings.rules[r.backend:upper()] = r
     end
   end
+  if settings.pusher_enabled.json_raw_tcp then
+    if not (settings.host and settings.port) then
+      rspamd_logger.errx(rspamd_config, 'No host and/or port is specified')
+      settings.pusher_enabled.json_raw_tcp = nil
+    else
+      local r = {}
+      r.backend = 'json_raw_tcp'
+      r.host = settings.host
+      r.port = settings.port
+      r.defer = settings.defer
+      r.selector = settings.pusher_select.json_raw_tcp
+      r.formatter = settings.pusher_format.json_raw_tcp
+      settings.rules[r.backend:upper()] = r
+    end
+  end
   if not next(settings.pusher_enabled) then
     rspamd_logger.errx(rspamd_config, 'No push backend enabled')
     return
@@ -530,6 +593,10 @@ local backend_required_elements = {
   redis_pubsub = {
     'channel',
   },
+  json_raw_tcp = {
+    'host',
+    'port',
+  },
 }
 local check_element = {
   selector = function(k, v)
@@ -579,6 +646,7 @@ backend_check.redis_pubsub = function(k, rule)
     settings.rules[k] = nil
   else
     backend_check.default(k, rule)
+    rule.timeout = redis_params.timeout
   end
 end
 setmetatable(backend_check, {
@@ -606,8 +674,10 @@ for k, v in pairs(settings.rules) do
 end
 
 local function gen_exporter(rule)
-  return function (task)
-    if task:has_flag('skip') then return end
+  return function(task)
+    if task:has_flag('skip') then
+      return
+    end
     local selector = rule.selector or 'default'
     local selected = selectors[selector](task)
     if selected then
@@ -634,7 +704,7 @@ for k, r in pairs(settings.rules) do
     name = 'EXPORT_METADATA_' .. k,
     type = 'idempotent',
     callback = gen_exporter(r),
-    priority = 10,
     flags = 'empty,explicit_disable,ignore_passthrough',
+    augmentations = { string.format("timeout=%f", r.timeout or settings.timeout or 0.0) }
   })
 end
