@@ -1124,7 +1124,7 @@ local function multimap_on_load_gen(rule)
         lua_util.debugm(N, rspamd_config, "%s: adding new symbol %s (score = %s), triggered by %s",
             rule.symbol, symbol, score, key)
         rspamd_config:register_symbol {
-          name = value,
+          name = symbol,
           parent = rule.callback_id,
           type = 'virtual',
           score = score,
@@ -1133,7 +1133,7 @@ local function multimap_on_load_gen(rule)
           group = N,
           score = 1.0, -- In future, we will parse score from `get_value` and use it as multiplier
           description = 'Automatic symbol generated by rule: ' .. rule.symbol,
-          name = value,
+          name = symbol,
         })
         known_symbols[value] = true
       end
